@@ -38,4 +38,7 @@ enum ofono_vendor {
 	OFONO_VENDOR_TELIT,
 	OFONO_VENDOR_SPEEDUP,
 	OFONO_VENDOR_SAMSUNG,
+	OFONO_VENDOR_SIMCOM,
+	OFONO_VENDOR_ICERA,
+	OFONO_VENDOR_WAVECOM_Q2XXX,
 };
