@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -28,7 +28,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
-#include <sys/stat.h>
 
 #include <glib.h>
 
@@ -42,8 +41,6 @@
 
 #include "huaweimodem.h"
 
-#define TUN_SYSFS_DIR "/sys/devices/virtual/misc/tun"
-
 static const char *none_prefix[] = { NULL };
 static const char *dhcp_prefix[] = { "^DHCP:", NULL };
 
@@ -75,40 +72,6 @@ static gboolean dhcp_poll(gpointer user_data)
 	return FALSE;
 }
 
-static gboolean ndis_receive_callback(GIOChannel *channel,
-					GIOCondition cond, gpointer user_data)
-{
-	struct ofono_gprs_context *gc = user_data;
-	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
-	GIOStatus status;
-	gsize bytes_read;
-	char buf[1059];
-
-	if (cond & (G_IO_NVAL | G_IO_ERR | G_IO_HUP)) {
-		gcd->ndis_watch = 0;
-		return FALSE;
-	}
-
-	status = g_io_channel_read_chars(channel, buf, sizeof(buf),
-							&bytes_read, NULL);
-
-	ofono_info("Received %zd bytes", bytes_read);
-
-	{
-		unsigned int i;
-		for (i = 0; i < bytes_read; i++)
-			printf("%02x ", buf[i]);
-		printf("\n");
-	}
-
-	if (status != G_IO_STATUS_NORMAL && status != G_IO_STATUS_AGAIN) {
-		gcd->ndis_watch = 0;
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 static gboolean get_next_addr(GAtResultIter *iter, char **addr)
 {
 	const char *str;
@@ -134,6 +97,7 @@ static void dhcp_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_gprs_context *gc = user_data;
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	GAtResultIter iter;
+	struct ofono_modem *modem;
 	const char *interface;
 	char *ip = NULL;
 	char *netmask = NULL;
@@ -141,9 +105,6 @@ static void dhcp_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	char *dns1 = NULL;
 	char *dns2 = NULL;
 	const char *dns[3];
-	struct ofono_modem *modem;
-	const char *devnode;
-	GIOChannel *channel;
 
 	DBG("ok %d", ok);
 
@@ -174,19 +135,7 @@ static void dhcp_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_info("DNS: %s, %s", dns1, dns2);
 
 	modem = ofono_gprs_context_get_modem(gc);
-	devnode = ofono_modem_get_string(modem, "NDIS");
-
-	ofono_info("NDIS: %s", devnode);
-
-	channel = g_at_tty_open(devnode, NULL);
-	if (channel) {
-		gcd->ndis_watch = g_io_add_watch(channel,
-				G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
-				ndis_receive_callback, gc);
-	}
-	g_io_channel_unref(channel);
-
-	interface = "invalid";
+	interface = ofono_modem_get_string(modem, "NetworkInterface");
 
 	ofono_gprs_context_set_interface(gc, interface);
 	ofono_gprs_context_set_ipv4_address(gc, ip, TRUE);
@@ -364,15 +313,9 @@ static int huawei_gprs_context_probe(struct ofono_gprs_context *gc,
 {
 	GAtChat *chat = data;
 	struct gprs_context_data *gcd;
-	struct stat st;
 
 	DBG("");
 
-	if (stat(TUN_SYSFS_DIR, &st) < 0) {
-		ofono_error("Missing support for TUN/TAP devices");
-		return -ENODEV;
-	}
-
 	gcd = g_try_new0(struct gprs_context_data, 1);
 	if (gcd == NULL)
 		return -ENOMEM;
