@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -40,8 +40,6 @@
 #include "ofono.h"
 #include "common.h"
 
-#define CALL_VOLUME_INTERFACE OFONO_SERVICE ".CallVolume"
-
 static GSList *g_drivers = NULL;
 
 struct ofono_call_volume {
@@ -67,7 +65,8 @@ void ofono_call_volume_set_speaker_volume(struct ofono_call_volume *cv,
 	if (__ofono_atom_get_registered(cv->atom) == FALSE)
 		return;
 
-	ofono_dbus_signal_property_changed(conn, path, CALL_VOLUME_INTERFACE,
+	ofono_dbus_signal_property_changed(conn, path,
+						OFONO_CALL_VOLUME_INTERFACE,
 						"SpeakerVolume",
 						DBUS_TYPE_BYTE, &percent);
 }
@@ -83,7 +82,8 @@ void ofono_call_volume_set_microphone_volume(struct ofono_call_volume *cv,
 	if (__ofono_atom_get_registered(cv->atom) == FALSE)
 		return;
 
-	ofono_dbus_signal_property_changed(conn, path, CALL_VOLUME_INTERFACE,
+	ofono_dbus_signal_property_changed(conn, path,
+						OFONO_CALL_VOLUME_INTERFACE,
 						"MicrophoneVolume",
 						DBUS_TYPE_BYTE, &percent);
 }
@@ -100,7 +100,8 @@ void ofono_call_volume_set_muted(struct ofono_call_volume *cv, int muted)
 		return;
 
 	m = muted;
-	ofono_dbus_signal_property_changed(conn, path, CALL_VOLUME_INTERFACE,
+	ofono_dbus_signal_property_changed(conn, path,
+						OFONO_CALL_VOLUME_INTERFACE,
 						"Muted", DBUS_TYPE_BOOLEAN, &m);
 }
 
@@ -113,8 +114,7 @@ static DBusMessage *cv_get_properties(DBusConnection *conn,
 	dbus_bool_t muted;
 
 	reply = dbus_message_new_method_return(msg);
-
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -154,7 +154,8 @@ static void sv_set_callback(const struct ofono_error *error, void *data)
 	__ofono_dbus_pending_reply(&cv->pending,
 				dbus_message_new_method_return(cv->pending));
 
-	ofono_dbus_signal_property_changed(conn, path, CALL_VOLUME_INTERFACE,
+	ofono_dbus_signal_property_changed(conn, path,
+					OFONO_CALL_VOLUME_INTERFACE,
 					"SpeakerVolume",
 					DBUS_TYPE_BYTE, &cv->speaker_volume);
 }
@@ -176,7 +177,8 @@ static void mv_set_callback(const struct ofono_error *error, void *data)
 	__ofono_dbus_pending_reply(&cv->pending,
 				dbus_message_new_method_return(cv->pending));
 
-	ofono_dbus_signal_property_changed(conn, path, CALL_VOLUME_INTERFACE,
+	ofono_dbus_signal_property_changed(conn, path,
+					OFONO_CALL_VOLUME_INTERFACE,
 					"MicrophoneVolume",
 					DBUS_TYPE_BYTE, &cv->microphone_volume);
 }
@@ -201,7 +203,8 @@ static void muted_set_callback(const struct ofono_error *error, void *data)
 	__ofono_dbus_pending_reply(&cv->pending,
 				dbus_message_new_method_return(cv->pending));
 
-	ofono_dbus_signal_property_changed(conn, path, CALL_VOLUME_INTERFACE,
+	ofono_dbus_signal_property_changed(conn, path,
+						OFONO_CALL_VOLUME_INTERFACE,
 						"Muted", DBUS_TYPE_BOOLEAN, &m);
 }
 
@@ -233,8 +236,8 @@ static DBusMessage *cv_set_property(DBusConnection *conn, DBusMessage *msg,
 	if (g_str_equal(property, "SpeakerVolume") == TRUE) {
 		unsigned char percent;
 
-		if (!cv->driver->speaker_volume)
-			return __ofono_error_not_supported(msg);
+		if (cv->driver->speaker_volume == NULL)
+			return __ofono_error_not_implemented(msg);
 
 		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_BYTE)
 			return __ofono_error_invalid_args(msg);
@@ -255,8 +258,8 @@ static DBusMessage *cv_set_property(DBusConnection *conn, DBusMessage *msg,
 	} else if (g_str_equal(property, "MicrophoneVolume") == TRUE) {
 		unsigned char percent;
 
-		if (!cv->driver->microphone_volume)
-			return __ofono_error_not_supported(msg);
+		if (cv->driver->microphone_volume == NULL)
+			return __ofono_error_not_implemented(msg);
 
 		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_BYTE)
 			return __ofono_error_invalid_args(msg);
@@ -277,8 +280,8 @@ static DBusMessage *cv_set_property(DBusConnection *conn, DBusMessage *msg,
 	} else if (g_str_equal(property, "Muted") == TRUE) {
 		dbus_bool_t muted;
 
-		if (!cv->driver->mute)
-			return __ofono_error_not_supported(msg);
+		if (cv->driver->mute == NULL)
+			return __ofono_error_not_implemented(msg);
 
 		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_BOOLEAN)
 			return __ofono_error_invalid_args(msg);
@@ -298,15 +301,19 @@ static DBusMessage *cv_set_property(DBusConnection *conn, DBusMessage *msg,
 	return __ofono_error_invalid_args(msg);
 }
 
-static GDBusMethodTable cv_methods[] = {
-	{ "GetProperties",	"",	"a{sv}",	cv_get_properties },
-	{ "SetProperty",	"sv",	"",		cv_set_property,
-							G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable cv_methods[] = {
+	{ GDBUS_METHOD("GetProperties",
+			NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+			cv_get_properties) },
+	{ GDBUS_ASYNC_METHOD("SetProperty",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" }),
+			NULL, cv_set_property) },
 	{ }
 };
 
-static GDBusSignalTable cv_signals[] = {
-	{ "PropertyChanged",	"sv" },
+static const GDBusSignalTable cv_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" })) },
 	{ }
 };
 
@@ -319,7 +326,7 @@ static void call_volume_remove(struct ofono_atom *atom)
 	if (cv == NULL)
 		return;
 
-	if (cv->driver && cv->driver->remove)
+	if (cv->driver != NULL && cv->driver->remove != NULL)
 		cv->driver->remove(cv);
 
 	g_free(cv);
@@ -366,9 +373,9 @@ static void call_volume_unregister(struct ofono_atom *atom)
 	struct ofono_modem *modem = __ofono_atom_get_modem(atom);
 	const char *path = __ofono_atom_get_path(atom);
 
-	ofono_modem_remove_interface(modem, CALL_VOLUME_INTERFACE);
+	ofono_modem_remove_interface(modem, OFONO_CALL_VOLUME_INTERFACE);
 	g_dbus_unregister_interface(conn, path,
-					CALL_VOLUME_INTERFACE);
+					OFONO_CALL_VOLUME_INTERFACE);
 }
 
 void ofono_call_volume_register(struct ofono_call_volume *cv)
@@ -378,16 +385,16 @@ void ofono_call_volume_register(struct ofono_call_volume *cv)
 	const char *path = __ofono_atom_get_path(cv->atom);
 
 	if (!g_dbus_register_interface(conn, path,
-					CALL_VOLUME_INTERFACE,
+					OFONO_CALL_VOLUME_INTERFACE,
 					cv_methods, cv_signals, NULL,
 					cv, NULL)) {
 		ofono_error("Could not create %s interface",
-					CALL_VOLUME_INTERFACE);
+					OFONO_CALL_VOLUME_INTERFACE);
 
 		return;
 	}
 
-	ofono_modem_add_interface(modem, CALL_VOLUME_INTERFACE);
+	ofono_modem_add_interface(modem, OFONO_CALL_VOLUME_INTERFACE);
 
 	__ofono_atom_register(cv->atom, call_volume_unregister);
 }
@@ -426,4 +433,3 @@ void *ofono_call_volume_get_data(struct ofono_call_volume *cv)
 {
 	return cv->driver_data;
 }
-
